const Handlebars = require('handlebars')

const utils = require('./utils')
const { compileTemplate, registerHelpers, registerPartials } = require('./handlebars')

/**
 * Creates a template compiler
 * @param {CompilerParams} globalParams
 */
function Compiler (globalParams = {}) {
  const hbs = Handlebars.create()

  const globalCss = utils.coerceArray(globalParams.css)
  const { helpers = {}, partials = {} } = globalParams

  registerHelpers(hbs, helpers)
  registerPartials(hbs, partials)

  /**
   * Compiles template
   * @param {CompilerInstanceParams} params
   */
  function compile (params) {
    const template = params.template
    const localCss = utils.coerceArray(params.css)
    const css = [...globalCss, ...localCss]

    const compiledTemplate = compileTemplate(hbs, { body: template, css })

    return globalParams.data
      ? data => compiledTemplate({ ...globalParams.data, ...data })
      : compiledTemplate
  }

  return compile
}

module.exports = Compiler

/**
 * @typedef {object} CompilerParams
 * @property {object} data global template data
 * @property {object} partials handlebars partials
 * @property {object} helpers handlebars helpers
 * @property {string|string[]} css global style string(s)
 */

/**
 * @typedef {object} CompilerInstanceParams
 * @property {string} template template string
 * @property {string|string[]} css local style string(s)
 */
