const fs = require('fs')
const { promisify } = require('util')

const readFile = promisify(fs.readFile)

readFile.sync = fs.readFileSync

const coerceArray = value => value === undefined
  ? []
  : Array.isArray(value)
    ? value
    : [value]

const compose = (...fns) => arg => fns.reduceRight(
  (arg, fn) => fn(arg),
  arg
)

const trace = (tag = '', transform = a => a) =>
  value => console.log(tag, transform(value)) || value

const eachObj = (fn, obj) => Object.keys(obj)
  .forEach(key => fn(key, obj[key], obj))

const replace = (re, substr) => str => str.replace(re, substr)

module.exports = {
  readFile,
  coerceArray,
  compose,
  eachObj,
  replace,
  trace
}
